-- (C) Copyright 2009, Zachary Scott
-- visit: http://cthug-zs.blogspot.com

-- enable usb
System.usbDiskModeActivate()

-- define some colors
red = Color.new(255, 0, 0)
green = Color.new(0, 255, 0)
blue = Color.new(0, 0, 255)
magicPink = Color.new(255, 0, 255)
black = Color.new(0, 0, 0)
white = Color.new(255, 255, 255)

-- init variables
dx = 0; dy = 0            -- analog stick pos
px = 16; py = 0           -- player pos
cx = 448; cy = 0          -- computer pos
by = 132; bx = 236        -- ball pos
ox = px                   -- old player x pos
cScore = 0; pScore = 0    -- scores
bSpeedX = 3; bSpeedY = 1  -- ball speed

-- load images
back = Image.load("back.png")
comPad = Image.load("comp.png")
playPad = Image.load("player.png")
ball = Image.load("ball.png")

--[[
 hBallMove: Handles ball movement
]]
function hBallMove()

	-- move ball
	bx = bx + bSpeedX
	by = by + bSpeedY

end

--[[
 hAnalogMove: Handles analog stick movement
]]
function hAnalogMove()

	-- find cursor position
	if math.abs(dx) > 32 then
		px = px + dx/156
	end
	if math.abs(dy) > 32 then
		py = py + dy/32
	end

end 

--[[
 hColl: Handles collision detection
]]
function hColl()

	-- player - ball
	if bx <= (px+playPad:width()) and bx >= px then
		if (by+ball:height()) >= py and by <= (py+playPad:height()) then
			if (by+ball:height()) <= py-8 or by >= (py+playPad:height())-8 then
				bSpeedY = bSpeedY * (-1)
			end
			bSpeedX = bSpeedX * (-1)
			if bx ~= ox then
				bx = bx + bSpeedX/2
				ox = bx
			end
		end
	end	
	 
	-- computer - ball
	if (bx+ball:width()) >= cx then
		if (by+ball:height()) >= cy and by <= (cy+comPad:height()) then
			if (by+ball:height()) <= cy-8 or by >= (cy+comPad:height())-8 then
				bSpeedY = bSpeedY * (-1)
			end
			bSpeedX = bSpeedX * (-1)
		end
	end	
	
	-- sides - ball
	if by <= 0 then
		bSpeedY = bSpeedY * (-1)
	end
	if by >= 272-ball:height() then
		bSpeedY = bSpeedY * (-1)
	end
	
	-- vertical walls - ball (for points)
	if bx <= 0 then
		bSpeedX = bSpeedX * (-1)
		bSpeedY = 1
		by = 132; bx = 236
		cScore = cScore + 1
	end
	if bx >= 480-ball:width() then
		bSpeedX = bSpeedX * (-1)
		bSpeedY = 1
		by = 132; bx = 236
		pScore = pScore + 1
	end
	
	-- player - walls
	if px <= 16 then
		px = 16
	end
	if px >= 200-playPad:width() then
		px = 200-playPad:width()
	end
	if py <= 0 then
		py = 0
	end
	if py >= 272-playPad:height() then
		py = 272-playPad:height()
	end
	
	-- computer - walls
	if cy <= 0 then
		cy = 0
	end
	if cy >= 272-comPad:height() then
		cy = 272-comPad:height()
	end

end

-- game loop
while true do
	
	-- read pad presses
	pad = Controls.read()
	
	-- draw background
	screen:blit(0, 0, back, 0, 0, back:width(), back:height(), false)
	
	-- handle ball movement
	hBallMove()
	
	-- handle player movement
	hAnalogMove()
	
	-- AI
	if bSpeedX > 0 then
		comSpeedY = (by - cy) * (bx / (480 * 2)) * (bx / (480 * 2))
		cy = cy + comSpeedY/2
	end
	
	-- collision detection and handling
	hColl()
	
	-- draw paddles and ball
	screen:blit(bx, by, ball)
	screen:blit(cx, cy, comPad)
	screen:blit(px, py, playPad)
	
	-- draw scores and info
	screen:print(130, 262, "http://cthug-zs.blogspot.com", white)
	screen:print(212, 0, pScore, blue)
	screen:print(238, 0, "|", white)
	screen:print(262 , 0, cScore, red)
	
	-- wait for vertical blank then swap buffers
	screen.waitVblankStart()
	screen.flip()
	
	-- handle controls
	if pad:start() then break end
	if pad:select() then screen:save("screenshot.tga") end
	dx = pad:analogX()
	dy = pad:analogY()

end

-- EOF
